--[[
************************************************************************
Project				: SKG
Author				: zhinjio
Project Revision	: r133
Project Date		: 20090128120151

File				: Events.lua
Commit Author		: zhinjio
Commit Revision		: 133
Commit Date			: 20090128120151
************************************************************************
Description	:
	Event handling

TODO		:
	
************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]
local MODNAME = "SKG"
local FULLNAME = MODNAME
local SKG = LibStub( "AceAddon-3.0" ):GetAddon( MODNAME, "AceEvent-3.0" )
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( MODNAME )

local addon = SKG

-- All Event Listeners to go here. Everything else should be loaded at this point
function addon:CHAT_MSG_WHISPER( eventName, arg1, sender )
	addon.Debug( "Received CHAT_MSG_WHISPER" )
	-- Handle incoming whispers for bid, suicide, etc
end

function addon:RAID_ROSTER_UPDATE( )
	addon.Debug( "Received RAID_ROSTER_UPDATE" )
	-- handle changes to the raid roster
end

function addon:ADDON_LOADED( eventName, arg1 )
	addon.Debug( "Received  ADDON_LOADED" )
	-- possibly handle other on loaded stuff
end

function addon:UNIT_SPELLCAST_SENT( eventName, spellCaster, spellName, blort, spellTarget )
	addon.Debug( "Received  UNIT_SPELLCAST_SENT" )
	-- I watch this because occasionally a loot event will be the opening of a chest
	-- or something else that does not translate into a mob with a GUID.
	--
	-- So if this spell being cast is "Opening", and the caster is "player", then
	-- save the target data so I reference it later in the LOOT OPENED event
end

function addon:LOOT_OPENED( )
	-- The player who is the master looter should get the goods
	addon.Debug( "Received LOOT_OPENED" )
end

function addon:GUILD_ROSTER_UPDATE( )
	-- In previous versions, this event would fire a few times right after addon init
	-- even though guild information was NOT yet available. It *does* appear now, however,
	-- that this information is available even on the very first of these events firing.
	-- Despite that, I'm still gonna wait for the 2nd fire of the event to do any real work.
	addon.Debug( "Received GUILD_ROSTER_UPDATE" )
	-- Can I see proper guild information yet?
	if ( addon.INIT_GUILD == 0 ) then
		-- Wait for the second fire
		addon.INIT_GUILD = 1
	elseif ( addon.INIT_GUILD == 1 ) then
		-- Ok, do the real work
		local inGuild = IsInGuild()
		if ( inGuild == nil ) then
			addon.Debug( "Not in guild. Don't INIT." )
		else
			local db = addon.db.profile
			-- Figure out our security level
			addon.SetSecurityLevel()
			-- create all our frames
			addon:CreateFrames()
			addon.MainFrame:SetScale( db.Options.General.uiscale )
			-- Event listeners
			addon:RegisterEvent( "ADDON_LOADED" )
			addon:RegisterEvent( "LOOT_OPENED" )
			addon:RegisterEvent( "RAID_ROSTER_UPDATE" )
			addon:RegisterEvent( "CHAT_MSG_WHISPER" )
			addon:RegisterEvent( "UNIT_SPELLCAST_SENT" )
			-- Other events to worry about sometime ??
			-- 		PARTY_LOOT_METHOD_CHANGED
			-- 		COMBAT_LOG_EVENT_UNFILTERED
			-- Comm registration (handlers are in Comms.lua)
			addon:RegisterComm( MODNAME )
			-- Register our chat command, and set its function
			addon:RegisterChatCommand( "skg", "OnSlashCommand" )
			-- If desired, spit out the load message
			if ( db.Options.General.suppressload == false ) then
				addon.Print( string.format( L["SKG_MSG_ADDON_LOADED"], addon.version ) )
			end
		end
		addon.INIT_GUILD = 2
	else
		-- We've already init'd. All I want to check is if the person is in guild. If they
		-- AREN'T, I should disable the addon.
		local inGuild = IsInGuild()
		if ( inGuild == nil ) then
			addon.Debug( "Not in guild. Disabling addon." )
			SKG.Disable()
		end
	end
end

--[[
************************************************************************
CHANGELOG:

Date : 1/25/09
	Rewrite
Date : 10/26/08
	Fixing Ace3 conversion errors. Moving RegisterEvent calls to OnInit
Date : 10/25/08
	Initial attempt with Ace3 instead of LibRock. Probably will require
	major edits once. Added header/footer. Changed the function definitions
	to only use what arguments they really need/use. 
************************************************************************
]]--
