--[[
************************************************************************
Project				: SKG
Author				: zhinjio
Project Revision	: r133
Project Date		: 20090128120151

File				: SKG_UtilityFunctions.lua
Commit Author		: zhinjio
Commit Revision		: 133
Commit Date			: 20090128120151
************************************************************************
Description	:

TODO		:

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]

local MODNAME = "SKG"
local FULLNAME = MODNAME
local SKG = LibStub( "AceAddon-3.0" ):GetAddon( MODNAME )
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( MODNAME )

local addon = SKG

-- Stolen blatantly from kgpanels. User for texture rotation
addon.angles = setmetatable({}, {
	__index = function(t, k)
		local k = floor(k % 360)
		if k == 0 then
			t[k] = {0,0,0,1,1,0,1,1}
			return t[k]
		end
		local A = cos(k)
		local B = sin(k)
		local ULx, ULy = (-0.5 * A - -0.5 * B) + 0.5, (-0.5 * B + -0.5 * A) + 0.5
		local LLx, LLy = (-0.5 * A - 0.5 * B) + 0.5, (-0.5 * B + 0.5 * A) + 0.5
		local URx, URy = (0.5 * A - -0.5 * B) + 0.5, (0.5 * B + -0.5 * A) + 0.5
		local LRx, LRy = (0.5 * A - 0.5 * B) + 0.5, (0.5 * B + 0.5 * A) + 0.5
		t[k] = {ULx,ULy,LLx,LLy,URx,URy,LRx,LRy}
		return t[k]
	end
})

local function splitColor( hexColor )
	local t = {}
	t.r = tonumber( string.match( hexColor, "(%x%x)%x%x%x%x" ), 16 )/255
	t.g = tonumber( string.match( hexColor, "%x%x(%x%x)%x%x" ), 16 )/255
	t.b = tonumber( string.match( hexColor, "%x%x%x%x(%x%x)" ), 16 )/255
	t.a = 1
	return t
end

function addon.Print( msg )
	local _, fh = DEFAULT_CHAT_FRAME:GetFont()
	fh = math.ceil( fh )
	local icon = "\124TInterface\\Addons\\" .. FULLNAME .. "\\icon:" .. fh .. "\124t"
	if ( msg ~= nil ) then
	   if( DEFAULT_CHAT_FRAME ) then
	      DEFAULT_CHAT_FRAME:AddMessage( icon .. " " .. MODNAME .. " : " .. msg, 0.6, 1.0, 1.0 )
	   end
	end
end

function addon.Debug( msg )
	local db = addon.db.profile
	if ( msg ~= nil ) then
		if ( db.Options.General.debug ) then
			addon.Print( "DEBUG: " .. msg )
		end
	end
end

function addon.myCreateFrame(
	frameName, parentFrame, sizeX, sizeY, clamped, hidden, 
	anchorFrom, anchorFrame, anchorTo, anchorOffsetX, anchorOffsetY,
	movable, strata, level, 
	bg, edge, bgColor, bgTrans, borderColor, borderTrans
)
	local newFrame
		newFrame = CreateFrame( "Frame", frameName, parentFrame )
		if ( hidden ) then
			newFrame:Hide()
		end
		newFrame:SetClampedToScreen( clamped )
		newFrame:SetHeight( sizeY )
		newFrame:SetWidth( sizeX )
		newFrame:SetPoint( anchorFrom, anchorFrame, anchorTo, anchorOffsetX, anchorOffsetY )
		newFrame:EnableMouse( movable )
		newFrame:SetMovable( movable )
		newFrame:SetFrameStrata( strata )
		newFrame:SetFrameLevel( level )
		-- and my backdrop
		if ( edge == "" ) then edge = "Interface\\Tooltips\\UI-Tooltip-Border" end
		if ( bg == "" ) then bg = "Interface\\ChatFrame\\ChatFrameBackground" end
		local bgFrame = {insets = {}}
			bgFrame.bgFile = bg
			bgFrame.edgeFile = edge
			bgFrame.tile = false
			bgFrame.tileSize = 0
			bgFrame.edgeSize = 16
			local inset = 4
			bgFrame.insets.left = inset
			bgFrame.insets.right = inset
			bgFrame.insets.top = inset
			bgFrame.insets.bottom = inset
			newFrame:SetBackdrop(bgFrame)
		-- and wrap it up
		local bgColorTable = splitColor( bgColor )
		local borderColorTable = splitColor( borderColor )
		newFrame:SetBackdropColor( bgColorTable.r, bgColorTable.g, bgColorTable.b, bgTrans )
		newFrame:SetBackdropBorderColor( borderColorTable.r, borderColorTable.g, borderColorTable.b, borderTrans )
	return newFrame
end

function addon.TooltipDisplay( this, textLabel, useTrans )
	this:SetScript( "OnEnter",
		function ( this )
			GameTooltip_SetDefaultAnchor( GameTooltip, this )
			if ( useTrans == 1 ) then
				GameTooltip:SetText( L[ textLabel ], HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b )
			else
				GameTooltip:SetText( textLabel, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b )
			end
			GameTooltip:Show()
		end
	)
	this:SetScript( "OnLeave",
		function( this )
			GameTooltip:Hide()
		end
	)
end

function addon.myCreateButton(
	bName, parentFrame,	bHeight, bWidth,
	anchorFrom, anchorFrame, anchorTo, xOffset, yOffset,
	bNormFont, bHighFont, initText, tAlign, tooltipText,
	TextureIndex, tRotate, frameLevel )

	-- TextureIndex:
	--		0 - normal button textures (UI-Panel-Button)
	--		1 - small arrow textures (UI-ChatIcon-ScrollDown)
	--
	-- tRotate ( rotate texture ):
	--		0 - no rotation
	--		1 - left 90 degrees
	--		2 - 180 degrees
	--		3 - right 90 degrees

	local button = CreateFrame( "Button", bName, parentFrame )
	button:SetWidth( bWidth )
	button:SetHeight( bHeight )
	button.rotate = tRotate * 90
	if ( frameLevel ~= 0 ) then
		button:SetFrameLevel( frameLevel )
	end

	-- Set our texture(s)
	if ( TextureIndex == 0 ) then
		local left = button:CreateTexture(button:GetName() .. "_LeftTexture", "BACKGROUND")
		button.left = left
		local middle = button:CreateTexture(button:GetName() .. "_MiddleTexture", "BACKGROUND")
		button.middle = middle
		local right = button:CreateTexture(button:GetName() .. "_RightTexture", "BACKGROUND")
		button.right = right

		left:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		right:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])

		left:SetPoint("TOPLEFT"); left:SetPoint("BOTTOMLEFT"); left:SetWidth(12)
		left:SetTexCoord(0, 0.09375, 0, 0.6875)
		right:SetPoint("TOPRIGHT"); right:SetPoint("BOTTOMRIGHT"); right:SetWidth(12)
		right:SetTexCoord(0.53125, 0.625, 0, 0.6875)
		middle:SetPoint("TOPLEFT", left, "TOPRIGHT"); middle:SetPoint("BOTTOMRIGHT", right, "BOTTOMLEFT")
		middle:SetTexCoord(0.09375, 0.53125, 0, 0.6875)
		
		local function button_OnMouseDown(this)
			if this:IsEnabled() == 1 then
				this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Down]])
				this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Down]])
				this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Down]])
			end
		end
		local function button_OnMouseUp(this)
			if this:IsEnabled() == 1 then
				this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
				this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
				this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
			end
		end
		local function button_Disable(this)
			this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Disabled]])
			this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Disabled]])
			this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Disabled]])
			this:__Disable()
			this:EnableMouse(false)
		end
		local function button_Enable(this)
			this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
			this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
			this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
			this:__Enable()
			this:EnableMouse(true)
		end

		button:SetScript("OnMouseDown", button_OnMouseDown)
		button:SetScript("OnMouseUp", button_OnMouseUp)
--		button:SetScript("OnEnter", SubControl_OnEnter)
--		button:SetScript("OnLeave", SubControl_OnLeave)
		button.__Enable = button.Enable; button.Enable = button_Enable
		button.__Disable = button.Disable; button.Disable = button_Disable

		local highlight = button:CreateTexture(button:GetName() .. "_Highlight", "OVERLAY", "UIPanelButtonHighlightTexture")
		button:SetHighlightTexture(highlight)
	elseif ( TextureIndex == 1 ) then
		local tex = button:CreateTexture( button:GetName() .. "_Texture", "BACKGROUND" )
		button.tex = tex
		tex:SetTexture( "Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up" )
		tex:SetAllPoints( button )
		tex:SetTexCoord(unpack(addon.angles[button.rotate]))
		
		local function button_OnMouseDown( this )
			if ( this:IsEnabled() == 1 ) then
				this.tex:SetTexture( "Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Down" )
				this.tex:SetTexCoord(unpack(addon.angles[this.rotate]))
			end
		end
		local function button_OnMouseUp( this )
		
			if ( this:IsEnabled() == 1 ) then
				this.tex:SetTexture( "Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up" )
				this.tex:SetTexCoord(unpack(addon.angles[this.rotate]))
			end
		end
		local function button_Disable( this )
			this.tex:SetTexture( "Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Disabled" )
			this.tex:SetTexCoord(unpack(addon.angles[this.rotate]))
			this:__Disable()
			this:EnableMouse( false )
		end
		local function button_Enable(this)
			this.tex:SetTexture( "Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up" )
			this.tex:SetTexCoord(unpack(addon.angles[this.rotate]))
			this:__Enable()
			this:EnableMouse( true )
		end
			
		button:SetScript( "OnMouseDown", button_OnMouseDown )
		button:SetScript( "OnMouseUp", button_OnMouseUp )
		button.__Enable = button.Enable; button.Enable = button_Enable
		button.__Disable = button.Disable; button.Disable = button_Disable
		button:SetHighlightTexture( "Interface\\ChatFrame\\UI-ChatIcon-BlinkHilight" )
	end
	-- text settings, if necessary
	if not ( initText == "" ) then
		local text = button:CreateFontString(button:GetName() .. "_FontString", "ARTWORK")
		button:SetFontString(text)
		button.text = text
		text:SetPoint("LEFT", button, "LEFT", 7, 0)
		text:SetPoint("RIGHT", button, "RIGHT", -7, 0)
		text:SetJustifyH( tAlign )
		button:SetNormalFontObject(bNormFont)
		button:SetHighlightFontObject(bHighFont)
		button:SetDisabledFontObject(GameFontDisableSmall)
		text:SetText( initText )		
	end
	-- set tooltip, if necessary
	if ( tooltipText ~= "" ) then
		addon.TooltipDisplay( button, tooltipText, 0 )
	end
	button:SetPoint( anchorFrom, anchorFrame, anchorTo, xOffset, yOffset )
	
	return button
end

function addon.CreateFrames()
	-- These frames are always created...
	addon.MainFrame_Create()
	-- addon.MainFrame

	-- And these we create if the right module is loaded
end

function addon.SetSecurityLevel()
-- Next thing to work on
-- For right now, just let the existing addon.SECURITY_LEVEL set in Globals.lua stand
end
--[[
************************************************************************
CHANGELOG:

Date : 1/25/09
	Rewrite
Date : 12/21/08
	Added in changes for Death Knight compatibility. I think I got
	everything.
Date : 10/25/08
	Initial attempt with Ace3 instead of LibRock. Probably will require
	major edits once. Added header/footer. Made changes to itemLinks
	that were updated with a new field in 3.0.2.
************************************************************************
]]--